/*********************************************************
* Project : CCIR Transmitter for Yaesu HM48 microphone   *
* Version : 1.0                                          *
* Date    : 05.03.2011                                   *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)               *
* Compiler: MPLAB + Hi-tech v.9.81                       *
**********************************************************/

#include <stdio.h>
#include <htc.h>

__CONFIG(WDTE_OFF & FOSC_XT & MCLRE_ON & BOREN_ON & LVP_OFF & CP_OFF); //Set fuses bits
#define _XTAL_FREQ 3686400 // Define Xtal freq
#define LEDred RB1 // Define Red LED as port RB1
#define LEDgreen RB2 // Define Green LED as port RB2
#define RL1 RB0 // Define relay 1 as port RB0
#define SW RB3 // Define pushswitch as port RB3
#define MPTT RB4 // Define MIC-switch as port RB4
#define tone RA2 //Output for CCIR tone
#define RL2 RB5 // Define PTT relay


unsigned char CCIR_key[11]; // define variable for CCIR input
unsigned char CCIR_tone[16]={112,199,188,176,164,155,148,136,127,120,92,244,98,228,105,213}; //define CCIR tones
//                            0   1   2   3   4   5   6   7   8   9   A  B   C   D   E   F
unsigned char antal_CCIR;

unsigned int AN0_input;
unsigned int AN1_input;

unsigned char prg_mode;


/*************************************************
* INITIALICE                                     *
* Setup PIC ports etc.                           *
**************************************************
* Version : 1.0                                  *
* Date    : 05.03.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/

void init(void)
{
	TRISB =0b00011000; // Set in- & output for port B
	TRISA =0b00000011; // Set in- & output for port A
	PORTA=0; // Clear all port A
	PORTB=0; // Clear all port B

	ADCON1 = 0b10000100; // config A/D converter
	OPTION_REG = 0b10000000; //Config timer0 and weak pullup PORT B off
	T1CON = 0b00111000; // config Timer1
	


	
	
}

/*************************************************
* READ ANALOG INPUTS                             *
* Reads input AN0 & AN1                          *
**************************************************
* Version : 1.0                                  *
* Date    : 05.03.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/

void AD_read(void)
{
	unsigned char x;
	unsigned int AN0_temp = 0;
	unsigned int AN1_temp = 0;

	ADCON0 = 0b01000001; // config A/D converter to port AN0
	for(x=0;x<10;x++) // read AN0 10 times
	{

		//Read Analog port 0

		GO_DONE = 1;
		while(GO_DONE){}
		AN0_input=(ADRESH<<8)+ADRESL;
		AN0_temp = AN0_temp + AN0_input;
	}

	ADCON0 = 0b01001001; // config A/D converter to port AN1
	for(x=0;x<10;x++) // read AN1 10 times
	{

		//Read Analog port 1


		GO_DONE = 1;
		while(GO_DONE){}
		AN1_input=(ADRESH<<8)+ADRESL;





		AN1_temp = AN1_temp + AN1_input;
	}
	AN0_input = AN0_temp/10; // find middel value from the 10 readings
	AN1_input = AN1_temp/10;
	ADON = 0; // turns of AD module
		
}

/*************************************************
* Send tone                                      *
* Makes the tone on PIN                          *
**************************************************
* Version : 1.0                                  *
* Date    : 08.03.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/

void send_tone(unsigned char time)
{
	TMR1IF = 0; //Clear timer 1 interrupt
	TMR1 = 0xFFFF - 0X2D00; // Set sending time to 100 ms (each tone is send for 100 ms)
	TMR1ON = 1;	//Starts timer1


	while(TMR1IF == 0)
	{

		LEDred=1;
		TMR0IF = 0;
		TMR0 = 0xff-time; // Set time for high  puls
		tone=1; //Set tone output high
		while (TMR0IF==0)
		{}
		tone=0; // Clear tone output
		TMR0IF = 0;
		TMR0 = (0xff-time)+11; //Set time for low  puls (+11 to correct for delay in rest of code)
		while (TMR0IF==0)
		{}

	}

	TMR1ON=0; //Stops timer1
	LEDred=0;
}


/*************************************************
* CCIR Send                                      *
* Generate CCIR tones                            *
**************************************************
* Version : 1.0                                  *
* Date    : 08.03.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/

void CCIR_send(void)
{
	unsigned char x;
	unsigned char temptime;
	RL2=1;	//Radio in Tx
	__delay_ms(500);	// Makes sure the radio is TXing before tones are send
	for(x=0;x<antal_CCIR;x++) // Loop for checking repeat af keys
	{

		if ((CCIR_key[x] == CCIR_key[x-1]) && (x >= 1))	// if key is the same as the key before, send E (repeat)
		{
			CCIR_key[x]=0xE;
		}
	}
	for(x=0;x<antal_CCIR;x++) // Loop for sending tones
	{
		temptime = CCIR_tone[CCIR_key[x]];

		send_tone(temptime);

	}
	RL2=0; // Radio in Rx
	__delay_ms(200);

}
/*************************************************
* Blink green                                    *
* flash the green LED for  sek                  *
* Sub routine for read key                       *
**************************************************
* Version : 1.0                                  *
* Date    : 08.03.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/

void blink_green(void)
{
	LEDgreen ^=1;
	__delay_ms(500);
	LEDgreen ^=1;
}

/*************************************************
* Read key                                       *
* REading the number keys                        *
**************************************************
* Version : 1.0                                  *
* Date    : 03.04.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/
void read_key(void)
{

	antal_CCIR=0;
	RL1 = 1;	// Set Relay 1 to read keys



	while(MPTT == 1 && SW == 0)	// Runs untill PTT key
	{
		AD_read();	// Read keys
		while(AN0_input > 920 && AN1_input > 920 && MPTT == 1 && SW == 0) // Wait on key
		{
			AD_read();
		}

	__delay_ms(100);	// wait to make sure key is pushed okay
	AD_read();

	if(AN0_input > 0 && AN0_input < 102) // If key value match col 1
	{
		if(AN1_input > 102 && AN1_input < 306) // If key value match "1"
		{
			CCIR_key[antal_CCIR] = 0x1;
			antal_CCIR++;
			blink_green();	// Flash green LED
		}
		else if(AN1_input > 306 && AN1_input < 510)  // If key value match "2"
		{
			CCIR_key[antal_CCIR] = 0x2;
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 510 && AN1_input < 714)  // If key value match "3"
		{
			CCIR_key[antal_CCIR] = 0x3;
			antal_CCIR++;
			blink_green();

		}
		else if(AN1_input > 714 && AN1_input < 918) // If key value match "A"
		{
			CCIR_key[antal_CCIR] = 0xA;
			antal_CCIR++;
			blink_green();

		}
	}
	else if(AN0_input > 102 && AN0_input < 306) // If key value match col 2
	{
		if(AN1_input > 102 && AN1_input < 306) // If key value match "4"
		{
			CCIR_key[antal_CCIR] = 0x4;
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 306 && AN1_input < 510) // If key value match "5"
		{
			CCIR_key[antal_CCIR] = 0x5;
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 510 && AN1_input < 714) // If key value match "6"
		{
			CCIR_key[antal_CCIR] = 0x6;
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 714 && AN1_input < 918) // If key value match "B"
		{
			CCIR_key[antal_CCIR] = 0xB;
			antal_CCIR++;
			blink_green();
		}
	}
	else if(AN0_input > 306 && AN0_input < 510) // If key value match col 3
	{
		if(AN1_input > 102 && AN1_input < 306) // If key value match "7"
		{
			CCIR_key[antal_CCIR] = 0x7;
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 306 && AN1_input < 510) // If key value match "8"
		{
			CCIR_key[antal_CCIR] = 0x8;
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 510 && AN1_input < 714) // If key value match "9"
		{
			CCIR_key[antal_CCIR] = 0x9;
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 714 && AN1_input < 918) // If key value match "C"
		{
			CCIR_key[antal_CCIR] = 0xC;
			antal_CCIR++;
			blink_green();
		}
	}	
	else if(AN0_input > 510 && AN0_input < 714) // If key value match col 4
	{
	if(AN1_input > 102 && AN1_input < 306)
		{
			CCIR_key[antal_CCIR] = 0xE; // If key value match "*"
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 306 && AN1_input < 510) // If key value match "0"
		{
			CCIR_key[antal_CCIR] = 0x0;
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 510 && AN1_input < 714)
		{
			CCIR_key[antal_CCIR] = 0xF; // If key value match "#"
			antal_CCIR++;
			blink_green();
		}
		else if(AN1_input > 714 && AN1_input < 918) // If key value match "D"
		{
			CCIR_key[antal_CCIR] = 0xD;
			antal_CCIR++;
			blink_green();
		}
	}

	while (AN0_input < 920 && AN1_input < 920)	//while untill key release
	{
		AD_read();
	}

	}
	if (SW == 1) // if botton is pushed for break
	{
		antal_CCIR = 0;
		while (SW == 1)
		{}
	}
	RL1 = 0; // Set relay 1 back to keys to radio


}

/*************************************************
* read mem                                       *
* Read the memories                              *
* If the first key is * it means that a memory   *
* callnumber is selected the next key selects it *
**************************************************
* Version : 1.0                                  *
* Date    : 05.04.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/

void read_mem(void)
{
	unsigned char mem_ard;
	unsigned char x;	

	mem_ard = CCIR_key[1]<<4; // Select memory adress
	antal_CCIR = eeprom_read(mem_ard); // read nr. of digis in memory
	for ( x=1;x<=antal_CCIR;x++)
	{
		CCIR_key[x-1] = eeprom_read(mem_ard+x); //read memory
	}

}


/*************************************************
* Prg mem                                        *
* Program the memories                           *
* Enter the memory number followed by the        *
* callnumber (like: xyyyyy)press PTT when done   *
**************************************************
* Version : 1.0                                  *
* Date    : 05.04.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/

void prg_mem(void)
{
	unsigned char mem_ard;
	unsigned char x;

	read_key();
	
	mem_ard = (CCIR_key[0]<<4);	// Select memory adress
	eeprom_write(mem_ard,antal_CCIR); // save nr. of digis in memory

	for(x=1;x<antal_CCIR;x++)
	{
		eeprom_write(mem_ard+x,CCIR_key[x]); // save memory
	}
}



/*************************************************
* Input                                          *
* Read and send input                            *
**************************************************
* Version : 1.0                                  *
* Date    : 05.04.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/

void input(void)
{
	unsigned char y = 0;
	prg_mode = 0;

	LEDgreen = 1; // turns on green LED
	while(SW == 1) //waits for the switch to be released
	{
		__delay_ms(10); // Makes a 0.01 sec. delay (to make the counter below smaller)
		y++;
		if (y>= 200) // check if programming mode is selected (switch pushed for more the 2 sec.)
		{
			LEDgreen = 0;
			LEDred = 1;
			prg_mode = 1;

		}
	
	}

	if(prg_mode == 0)
	{
		read_key();
		LEDgreen = 0;
		if (antal_CCIR > 0) // if digis is pushed
		{
			if (CCIR_key[0] == 0xE)	// memory selected
			{
				read_mem(); // read memory
			}
		
		CCIR_send(); // Send tones
		}
	}
	else
	{
		prg_mem(); // programming mode
			LEDred = 0;
			LEDgreen = 1;	// show green LED to confirm programming ending
			__delay_ms(1000);
			LEDgreen = 0;

	}

}


/*************************************************
* Main                                           *
* Main loop                                      *
**************************************************
* Version : 1.0                                  *
* Date    : 08.03.2011                           *
* Designer: Niels Krogsgaard, OZ7ADZ(5P6Q)       *
**************************************************/
void main(void)
{
	init();
	while(1)
	{
		if(MPTT == 0)	//PTT Control in standby mode
		{
			RL2 = 1;
		}
		else
		{
			RL2 = 0;
		}

		if(SW == 1)		// CCIR swtich
		{
			input(); // key input rutine
  		}      

	}
}

